@@ -35,7 +35,7 @@ import java.util.Collections;
 //
 
 class VncCanvas extends Canvas
-  implements KeyListener, MouseListener, MouseMotionListener {
+  implements KeyListener, MouseListener, MouseMotionListener, MouseWheelListener {
 
   VncViewer viewer;
   RfbProto rfb;
@@ -184,6 +184,7 @@ class VncCanvas extends Canvas
       inputEnabled = true;
       addMouseListener(this);
       addMouseMotionListener(this);
+      addMouseWheelListener(this);
       if (viewer.showControls) {
 	viewer.buttonPanel.enableRemoteAccessControls(true);
       }
@@ -192,6 +193,7 @@ class VncCanvas extends Canvas
       inputEnabled = false;
       removeMouseListener(this);
       removeMouseMotionListener(this);
+      removeMouseWheelListener(this);
       if (viewer.showControls) {
 	viewer.buttonPanel.enableRemoteAccessControls(false);
       }
@@ -1205,6 +1207,9 @@ class VncCanvas extends Canvas
   public void mouseDragged(MouseEvent evt) {
     processLocalMouseEvent(evt, true);
   }
+  public void mouseWheelMoved(MouseWheelEvent evt) {
+    processLocalMouseWheelEvent(evt);
+  }
 
   public void processLocalKeyEvent(KeyEvent evt) {
     if (viewer.rfb != null && rfb.inNormalProtocol) {
@@ -1236,6 +1241,19 @@ class VncCanvas extends Canvas
     evt.consume();
   }
 
+  public void processLocalMouseWheelEvent(MouseWheelEvent evt) {
+    if (viewer.rfb != null && rfb.inNormalProtocol) {
+      synchronized(rfb) {
+	try {
+	  rfb.writeWheelEvent(evt);
+	} catch (Exception e) {
+	  e.printStackTrace();
+	}
+	rfb.notify();
+      }
+    }
+  }
+
   public void processLocalMouseEvent(MouseEvent evt, boolean moved) {
     if (viewer.rfb != null && rfb.inNormalProtocol) {
       if (moved) {
