@@ -103,6 +103,9 @@ public class VncViewer extends java.applet.Applet
   boolean trustAllVncCerts;
   boolean trustUrlVncCert;
   boolean debugCerts;
+  boolean debugKeyboard;
+  boolean mapF5_to_atsign;
+  boolean forbid_Ctrl_Alt;
 
   // Reference to this applet for inter-applet communication.
   public static java.applet.Applet refApplet;
@@ -778,6 +781,21 @@ public class VncViewer extends java.applet.Applet
     if (str != null && str.equalsIgnoreCase("Yes")) {
 	debugCerts = true;
     }
+    debugKeyboard = false;
+    str = readParameter("debugKeyboard", false);
+    if (str != null && str.equalsIgnoreCase("Yes")) {
+	debugKeyboard = true;
+    }
+    mapF5_to_atsign = false;
+    str = readParameter("mapF5_to_atsign", false);
+    if (str != null && str.equalsIgnoreCase("Yes")) {
+	mapF5_to_atsign = true;
+    }
+    forbid_Ctrl_Alt = false;
+    str = readParameter("forbid_Ctrl_Alt", false);
+    if (str != null && str.equalsIgnoreCase("Yes")) {
+	forbid_Ctrl_Alt = true;
+    }
   }
 
   public String readParameter(String name, boolean required) {
