@@ -19,6 +19,7 @@
  */
 
 /*
+ *  Copyright (C) 2005 Rohit Kumar, Johannes E. Schindelin
  *  OSXvnc Copyright (C) 2001 Dan McGuirk <mcguirk@incompleteness.net>.
  *  Original Xvnc code Copyright (C) 1999 AT&T Laboratories Cambridge.  
  *  All Rights Reserved.
@@ -45,14 +46,6 @@
 #include <sys/types.h>
 #endif
 
-#ifdef WIN32
-#pragma warning (disable: 4018 4761)
-#define close closesocket
-#define read(sock,buf,len) recv(sock,buf,len,0)
-#define EWOULDBLOCK WSAEWOULDBLOCK
-#define ETIMEDOUT WSAETIMEDOUT
-#define write(sock,buf,len) send(sock,buf,len,0)
-#else
 #ifdef LIBVNCSERVER_HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
@@ -65,10 +58,12 @@
 #include <netdb.h>
 #include <arpa/inet.h>
 #endif
+#ifdef LIBVNCSERVER_HAVE_SYS_UN_H
+#include <sys/un.h>
+#endif
 #ifdef LIBVNCSERVER_HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#endif
 
 #if defined(__linux__) && defined(NEED_TIMEVAL)
 struct timeval 
@@ -91,10 +86,17 @@ int allow_severity=LOG_INFO;
 int deny_severity=LOG_WARNING;
 #endif
 
-/*#ifndef WIN32
-int max(int i,int j) { return(i<j?j:i); }
+#if defined(WIN32)
+#ifndef __MINGW32__
+#pragma warning (disable: 4018 4761)
+#endif
+#define read(sock,buf,len) recv(sock,buf,len,0)
+#define EWOULDBLOCK WSAEWOULDBLOCK
+#define ETIMEDOUT WSAETIMEDOUT
+#define write(sock,buf,len) send(sock,buf,len,0)
+#else
+#define closesocket close
 #endif
-*/
 
 int rfbMaxClientWait = 20000;   /* time (ms) after which we decide client has
                                    gone away - needed to stop us hanging */
@@ -107,26 +109,19 @@ int rfbMaxClientWait = 20000;   /* time (ms) after which we decide client has
 void
 rfbInitSockets(rfbScreenInfoPtr rfbScreen)
 {
-    if (rfbScreen->socketInitDone)
+    in_addr_t iface = rfbScreen->listenInterface;
+
+    if (rfbScreen->socketState!=RFB_SOCKET_INIT)
 	return;
 
-    rfbScreen->socketInitDone = TRUE;
+    rfbScreen->socketState = RFB_SOCKET_READY;
 
     if (rfbScreen->inetdSock != -1) {
-	const int one = 1;
-
-#ifndef WIN32
-	if (fcntl(rfbScreen->inetdSock, F_SETFL, O_NONBLOCK) < 0) {
-	    rfbLogPerror("fcntl");
+        if(!rfbSetNonBlocking(rfbScreen->inetdSock))
 	    return;
-	}
-#endif
 
-	if (setsockopt(rfbScreen->inetdSock, IPPROTO_TCP, TCP_NODELAY,
-		       (char *)&one, sizeof(one)) < 0) {
-	    rfbLogPerror("setsockopt");
+        if(!rfbSetTcpNoDelay(rfbScreen->inetdSock))
 	    return;
-	}
 
     	FD_ZERO(&(rfbScreen->allFds));
     	FD_SET(rfbScreen->inetdSock, &(rfbScreen->allFds));
@@ -137,10 +132,9 @@ rfbInitSockets(rfbScreenInfoPtr rfbScreen)
     if(rfbScreen->autoPort) {
         int i;
         rfbLog("Autoprobing TCP port \n");
-
         for (i = 5900; i < 6000; i++) {
-            if ((rfbScreen->rfbListenSock = ListenOnTCPPort(i)) >= 0) {
-		rfbScreen->rfbPort = i;
+            if ((rfbScreen->listenSock = rfbListenOnTCPPort(i, iface)) >= 0) {
+		rfbScreen->port = i;
 		break;
 	    }
         }
@@ -150,28 +144,28 @@ rfbInitSockets(rfbScreenInfoPtr rfbScreen)
 	    return;
         }
 
-        rfbLog("Autoprobing selected port %d\n", rfbScreen->rfbPort);
+        rfbLog("Autoprobing selected port %d\n", rfbScreen->port);
         FD_ZERO(&(rfbScreen->allFds));
-        FD_SET(rfbScreen->rfbListenSock, &(rfbScreen->allFds));
-        rfbScreen->maxFd = rfbScreen->rfbListenSock;
+        FD_SET(rfbScreen->listenSock, &(rfbScreen->allFds));
+        rfbScreen->maxFd = rfbScreen->listenSock;
     }
-    else if(rfbScreen->rfbPort>0) {
-      rfbLog("Listening for VNC connections on TCP port %d\n", rfbScreen->rfbPort);
+    else if(rfbScreen->port>0) {
+      rfbLog("Listening for VNC connections on TCP port %d\n", rfbScreen->port);
 
-      if ((rfbScreen->rfbListenSock = ListenOnTCPPort(rfbScreen->rfbPort)) < 0) {
+      if ((rfbScreen->listenSock = rfbListenOnTCPPort(rfbScreen->port, iface)) < 0) {
 	rfbLogPerror("ListenOnTCPPort");
 	return;
       }
 
       FD_ZERO(&(rfbScreen->allFds));
-      FD_SET(rfbScreen->rfbListenSock, &(rfbScreen->allFds));
-      rfbScreen->maxFd = rfbScreen->rfbListenSock;
+      FD_SET(rfbScreen->listenSock, &(rfbScreen->allFds));
+      rfbScreen->maxFd = rfbScreen->listenSock;
     }
 
     if (rfbScreen->udpPort != 0) {
 	rfbLog("rfbInitSockets: listening for input on UDP port %d\n",rfbScreen->udpPort);
 
-	if ((rfbScreen->udpSock = ListenOnUDPPort(rfbScreen->udpPort)) < 0) {
+	if ((rfbScreen->udpSock = rfbListenOnUDPPort(rfbScreen->udpPort, iface)) < 0) {
 	    rfbLogPerror("ListenOnUDPPort");
 	    return;
 	}
@@ -180,6 +174,31 @@ rfbInitSockets(rfbScreenInfoPtr rfbScreen)
     }
 }
 
+void rfbShutdownSockets(rfbScreenInfoPtr rfbScreen)
+{
+    if (rfbScreen->socketState!=RFB_SOCKET_READY)
+	return;
+
+    rfbScreen->socketState = RFB_SOCKET_SHUTDOWN;
+
+    if(rfbScreen->inetdSock>-1) {
+	closesocket(rfbScreen->inetdSock);
+	FD_CLR(rfbScreen->inetdSock,&rfbScreen->allFds);
+	rfbScreen->inetdSock=-1;
+    }
+
+    if(rfbScreen->listenSock>-1) {
+	closesocket(rfbScreen->listenSock);
+	FD_CLR(rfbScreen->listenSock,&rfbScreen->allFds);
+	rfbScreen->listenSock=-1;
+    }
+
+    if(rfbScreen->udpSock>-1) {
+	closesocket(rfbScreen->udpSock);
+	FD_CLR(rfbScreen->udpSock,&rfbScreen->allFds);
+	rfbScreen->udpSock=-1;
+    }
+}
 
 /*
  * rfbCheckFds is called from ProcessInputEvents to check for input on the RFB
@@ -188,127 +207,153 @@ rfbInitSockets(rfbScreenInfoPtr rfbScreen)
  * rfbProcessClientMessage, etc).
  */
 
-void
+int
 rfbCheckFds(rfbScreenInfoPtr rfbScreen,long usec)
 {
     int nfds;
     fd_set fds;
     struct timeval tv;
     struct sockaddr_in addr;
-    size_t addrlen = sizeof(addr);
+    socklen_t addrlen = sizeof(addr);
     char buf[6];
-    const int one = 1;
     int sock;
     rfbClientIteratorPtr i;
     rfbClientPtr cl;
+    int result = 0;
 
     if (!rfbScreen->inetdInitDone && rfbScreen->inetdSock != -1) {
 	rfbNewClientConnection(rfbScreen,rfbScreen->inetdSock); 
 	rfbScreen->inetdInitDone = TRUE;
     }
 
-    memcpy((char *)&fds, (char *)&(rfbScreen->allFds), sizeof(fd_set));
-    tv.tv_sec = 0;
-    tv.tv_usec = usec;
-    nfds = select(rfbScreen->maxFd + 1, &fds, NULL, NULL /* &fds */, &tv);
-    if (nfds == 0) {
-	return;
-    }
-    if (nfds < 0) {
+    do {
+	memcpy((char *)&fds, (char *)&(rfbScreen->allFds), sizeof(fd_set));
+	tv.tv_sec = 0;
+	tv.tv_usec = usec;
+	nfds = select(rfbScreen->maxFd + 1, &fds, NULL, NULL /* &fds */, &tv);
+	if (nfds == 0) {
+	    /* timed out, check for async events */
+            i = rfbGetClientIterator(rfbScreen);
+            while((cl = rfbClientIteratorNext(i))) {
+                if (cl->onHold)
+                    continue;
+                if (FD_ISSET(cl->sock, &(rfbScreen->allFds)))
+                    rfbSendFileTransferChunk(cl);
+            }
+            rfbReleaseClientIterator(i);
+	    return result;
+	}
+
+	if (nfds < 0) {
 #ifdef WIN32
-		errno = WSAGetLastError();
+	    errno = WSAGetLastError();
 #endif
-	if (errno != EINTR)
+	    if (errno != EINTR)
 		rfbLogPerror("rfbCheckFds: select");
-	return;
-    }
-
-    if (rfbScreen->rfbListenSock != -1 && FD_ISSET(rfbScreen->rfbListenSock, &fds)) {
-
-	if ((sock = accept(rfbScreen->rfbListenSock,
-			   (struct sockaddr *)&addr, &addrlen)) < 0) {
-	    rfbLogPerror("rfbCheckFds: accept");
-	    return;
+	    return -1;
 	}
 
-#ifndef WIN32
-	if (fcntl(sock, F_SETFL, O_NONBLOCK) < 0) {
-	    rfbLogPerror("rfbCheckFds: fcntl");
-	    close(sock);
-	    return;
-	}
-#endif
+	result += nfds;
 
-	if (setsockopt(sock, IPPROTO_TCP, TCP_NODELAY,
-		       (char *)&one, sizeof(one)) < 0) {
-	    rfbLogPerror("rfbCheckFds: setsockopt");
-	    close(sock);
-	    return;
-	}
+	if (rfbScreen->listenSock != -1 && FD_ISSET(rfbScreen->listenSock, &fds)) {
+	    rfbBool sockIsInet = FALSE;
 
-#ifdef USE_LIBWRAP
-	if(!hosts_ctl("vnc",STRING_UNKNOWN,inet_ntoa(addr.sin_addr),
-		      STRING_UNKNOWN)) {
-	  rfbLog("Rejected connection from client %s\n",
-		 inet_ntoa(addr.sin_addr));
-	  close(sock);
-	  return;
-	}
-#endif
+	    memset(&addr, 0, sizeof(addr));
+	    addrlen = sizeof(addr);
 
-	rfbLog("Got connection from client %s\n", inet_ntoa(addr.sin_addr));
-
-	rfbNewClient(rfbScreen,sock);
-	
-	FD_CLR(rfbScreen->rfbListenSock, &fds);
-	if (--nfds == 0)
-	    return;
-    }
-
-    if ((rfbScreen->udpSock != -1) && FD_ISSET(rfbScreen->udpSock, &fds)) {
-        if(!rfbScreen->udpClient)
-	    rfbNewUDPClient(rfbScreen);
-	if (recvfrom(rfbScreen->udpSock, buf, 1, MSG_PEEK,
-		     (struct sockaddr *)&addr, &addrlen) < 0) {
-	    rfbLogPerror("rfbCheckFds: UDP: recvfrom");
-	    rfbDisconnectUDPSock(rfbScreen);
-	    rfbScreen->udpSockConnected = FALSE;
-	} else {
-	    if (!rfbScreen->udpSockConnected ||
-		(memcmp(&addr, &rfbScreen->udpRemoteAddr, addrlen) != 0))
-	    {
-		/* new remote end */
-		rfbLog("rfbCheckFds: UDP: got connection\n");
-
-		memcpy(&rfbScreen->udpRemoteAddr, &addr, addrlen);
-		rfbScreen->udpSockConnected = TRUE;
-
-		if (connect(rfbScreen->udpSock,
-			    (struct sockaddr *)&addr, addrlen) < 0) {
-		    rfbLogPerror("rfbCheckFds: UDP: connect");
-		    rfbDisconnectUDPSock(rfbScreen);
-		    return;
-		}
-
-		rfbNewUDPConnection(rfbScreen,rfbScreen->udpSock);
+	    if ((sock = accept(rfbScreen->listenSock,
+			    (struct sockaddr *)&addr, &addrlen)) < 0) {
+		rfbLogPerror("rfbCheckFds: accept");
+		return -1;
 	    }
 
-	    rfbProcessUDPInput(rfbScreen);
+	    if (addr.sin_family == AF_INET) {
+		sockIsInet = TRUE;
+	    }
+
+	    if(!rfbSetNonBlocking(sock)) {
+		closesocket(sock);
+		return -1;
+	    }
+
+	    if(!rfbSetTcpNoDelay(sock)) {
+		closesocket(sock);
+		return -1;
+	    }
+
+#ifdef USE_LIBWRAP
+	    if(sockIsInet && !hosts_ctl("vnc",STRING_UNKNOWN,inet_ntoa(addr.sin_addr),
+			STRING_UNKNOWN)) {
+		rfbLog("Rejected connection from client %s\n",
+			inet_ntoa(addr.sin_addr));
+		closesocket(sock);
+		return -1;
+	    }
+#endif
+
+	    rfbLog("Got connection from client %s\n",
+		sockIsInet ? inet_ntoa(addr.sin_addr) : "non-AF_INET-socket");
+
+	    rfbNewClient(rfbScreen,sock);
+
+	    FD_CLR(rfbScreen->listenSock, &fds);
+	    if (--nfds == 0)
+		return result;
 	}
 
-	FD_CLR(rfbScreen->udpSock, &fds);
-	if (--nfds == 0)
-	    return;
-    }
+	if ((rfbScreen->udpSock != -1) && FD_ISSET(rfbScreen->udpSock, &fds)) {
+	    if(!rfbScreen->udpClient)
+		rfbNewUDPClient(rfbScreen);
+	    if (recvfrom(rfbScreen->udpSock, buf, 1, MSG_PEEK,
+			(struct sockaddr *)&addr, &addrlen) < 0) {
+		rfbLogPerror("rfbCheckFds: UDP: recvfrom");
+		rfbDisconnectUDPSock(rfbScreen);
+		rfbScreen->udpSockConnected = FALSE;
+	    } else {
+		if (!rfbScreen->udpSockConnected ||
+			(memcmp(&addr, &rfbScreen->udpRemoteAddr, addrlen) != 0))
+		{
+		    /* new remote end */
+		    rfbLog("rfbCheckFds: UDP: got connection\n");
 
-    i = rfbGetClientIterator(rfbScreen);
-    while((cl = rfbClientIteratorNext(i))) {
-      if (cl->onHold)
-	continue;
-      if (FD_ISSET(cl->sock, &fds) && FD_ISSET(cl->sock, &(rfbScreen->allFds)))
-	rfbProcessClientMessage(cl);
-    }
-    rfbReleaseClientIterator(i);
+		    memcpy(&rfbScreen->udpRemoteAddr, &addr, addrlen);
+		    rfbScreen->udpSockConnected = TRUE;
+
+		    if (connect(rfbScreen->udpSock,
+				(struct sockaddr *)&addr, addrlen) < 0) {
+			rfbLogPerror("rfbCheckFds: UDP: connect");
+			rfbDisconnectUDPSock(rfbScreen);
+			return -1;
+		    }
+
+		    rfbNewUDPConnection(rfbScreen,rfbScreen->udpSock);
+		}
+
+		rfbProcessUDPInput(rfbScreen);
+	    }
+
+	    FD_CLR(rfbScreen->udpSock, &fds);
+	    if (--nfds == 0)
+		return result;
+	}
+
+	i = rfbGetClientIterator(rfbScreen);
+	while((cl = rfbClientIteratorNext(i))) {
+
+	    if (cl->onHold)
+		continue;
+
+            if (FD_ISSET(cl->sock, &(rfbScreen->allFds)))
+            {
+                if (FD_ISSET(cl->sock, &fds))
+                    rfbProcessClientMessage(cl);
+                else
+                    rfbSendFileTransferChunk(cl);
+            }
+	}
+	rfbReleaseClientIterator(i);
+    } while(rfbScreen->handleEventsEagerly);
+    return result;
 }
 
 
@@ -321,9 +366,14 @@ rfbDisconnectUDPSock(rfbScreenInfoPtr rfbScreen)
 
 
 void
-rfbCloseClient(cl)
-     rfbClientPtr cl;
+rfbCloseClient(rfbClientPtr cl)
 {
+    rfbExtensionData* extension;
+
+    for(extension=cl->extensions; extension; extension=extension->next)
+	if(extension->extension->close)
+	    extension->extension->close(cl, extension->data);
+
     LOCK(cl->updateMutex);
 #ifdef LIBVNCSERVER_HAVE_LIBPTHREAD
     if (cl->sock != -1)
@@ -334,8 +384,10 @@ rfbCloseClient(cl)
 	  while(cl->screen->maxFd>0
 		&& !FD_ISSET(cl->screen->maxFd,&(cl->screen->allFds)))
 	    cl->screen->maxFd--;
+#ifndef __MINGW32__
 	shutdown(cl->sock,SHUT_RDWR);
-	close(cl->sock);
+#endif
+	closesocket(cl->sock);
 	cl->sock = -1;
       }
     TSIGNAL(cl->updateCond);
@@ -348,34 +400,27 @@ rfbCloseClient(cl)
  */
 
 int
-rfbConnect(rfbScreen, host, port)
-    rfbScreenInfoPtr rfbScreen;
-    char *host;
-    int port;
+rfbConnect(rfbScreenInfoPtr rfbScreen,
+           char *host,
+           int port)
 {
     int sock;
-    int one = 1;
 
     rfbLog("Making connection to client on host %s port %d\n",
 	   host,port);
 
-    if ((sock = ConnectToTcpAddr(host, port)) < 0) {
+    if ((sock = rfbConnectToTcpAddr(host, port)) < 0) {
 	rfbLogPerror("connection failed");
 	return -1;
     }
 
-#ifndef WIN32
-    if (fcntl(sock, F_SETFL, O_NONBLOCK) < 0) {
-	rfbLogPerror("fcntl failed");
-	close(sock);
+    if(!rfbSetNonBlocking(sock)) {
+        closesocket(sock);
 	return -1;
     }
-#endif
 
-    if (setsockopt(sock, IPPROTO_TCP, TCP_NODELAY,
-		   (char *)&one, sizeof(one)) < 0) {
-	rfbLogPerror("setsockopt failed");
-	close(sock);
+    if(!rfbSetTcpNoDelay(sock)) {
+        closesocket(sock);
 	return -1;
     }
 
@@ -393,7 +438,7 @@ rfbConnect(rfbScreen, host, port)
  */
 
 int
-ReadExactTimeout(rfbClientPtr cl, char* buf, int len, int timeout)
+rfbReadExactTimeout(rfbClientPtr cl, char* buf, int len, int timeout)
 {
     int sock = cl->sock;
     int n;
@@ -414,11 +459,14 @@ ReadExactTimeout(rfbClientPtr cl, char* buf, int len, int timeout)
 
         } else {
 #ifdef WIN32
-			errno = WSAGetLastError();
+	    errno = WSAGetLastError();
 #endif
 	    if (errno == EINTR)
 		continue;
 
+#ifdef LIBVNCSERVER_ENOENT_WORKAROUND
+	    if (errno != ENOENT)
+#endif
             if (errno != EWOULDBLOCK && errno != EAGAIN) {
                 return n;
             }
@@ -438,12 +486,24 @@ ReadExactTimeout(rfbClientPtr cl, char* buf, int len, int timeout)
             }
         }
     }
+#undef DEBUG_READ_EXACT
+#ifdef DEBUG_READ_EXACT
+    rfbLog("ReadExact %d bytes\n",len);
+    for(n=0;n<len;n++)
+	    fprintf(stderr,"%02x ",(unsigned char)buf[n]);
+    fprintf(stderr,"\n");
+#endif
+
     return 1;
 }
 
-int ReadExact(rfbClientPtr cl,char* buf,int len)
+int rfbReadExact(rfbClientPtr cl,char* buf,int len)
 {
-  return(ReadExactTimeout(cl,buf,len,rfbMaxClientWait));
+  /* favor the per-screen value if set */
+  if(cl->screen && cl->screen->maxClientWait)
+    return(rfbReadExactTimeout(cl,buf,len,cl->screen->maxClientWait));
+  else
+    return(rfbReadExactTimeout(cl,buf,len,rfbMaxClientWait));
 }
 
 /*
@@ -453,16 +513,16 @@ int ReadExact(rfbClientPtr cl,char* buf,int len)
  */
 
 int
-WriteExact(cl, buf, len)
-     rfbClientPtr cl;
-     const char *buf;
-     int len;
+rfbWriteExact(rfbClientPtr cl,
+              const char *buf,
+              int len)
 {
     int sock = cl->sock;
     int n;
     fd_set fds;
     struct timeval tv;
     int totalTimeWaited = 0;
+    const int timeout = (cl->screen && cl->screen->maxClientWait) ? cl->screen->maxClientWait : rfbMaxClientWait;
 
 #undef DEBUG_WRITE_EXACT
 #ifdef DEBUG_WRITE_EXACT
@@ -498,7 +558,7 @@ WriteExact(cl, buf, len)
                 return n;
             }
 
-            /* Retry every 5 seconds until we exceed rfbMaxClientWait.  We
+            /* Retry every 5 seconds until we exceed timeout.  We
                need to do this because select doesn't necessarily return
                immediately when the other end has gone away */
 
@@ -516,7 +576,7 @@ WriteExact(cl, buf, len)
             }
             if (n == 0) {
                 totalTimeWaited += 5000;
-                if (totalTimeWaited >= rfbMaxClientWait) {
+                if (totalTimeWaited >= timeout) {
                     errno = ETIMEDOUT;
                     UNLOCK(cl->outputMutex);
                     return -1;
@@ -530,9 +590,28 @@ WriteExact(cl, buf, len)
     return 1;
 }
 
+/* currently private, called by rfbProcessArguments() */
 int
-ListenOnTCPPort(port)
-    int port;
+rfbStringToAddr(char *str, in_addr_t *addr)  {
+    if (str == NULL || *str == '\0' || strcmp(str, "any") == 0) {
+        *addr = htonl(INADDR_ANY);
+    } else if (strcmp(str, "localhost") == 0) {
+        *addr = htonl(INADDR_LOOPBACK);
+    } else {
+        struct hostent *hp;
+        if ((*addr = inet_addr(str)) == htonl(INADDR_NONE)) {
+            if (!(hp = gethostbyname(str))) {
+                return 0;
+            }
+            *addr = *(unsigned long *)hp->h_addr;
+        }
+    }
+    return 1;
+}
+
+int
+rfbListenOnTCPPort(int port,
+                   in_addr_t iface)
 {
     struct sockaddr_in addr;
     int sock;
@@ -541,23 +620,22 @@ ListenOnTCPPort(port)
     memset(&addr, 0, sizeof(addr));
     addr.sin_family = AF_INET;
     addr.sin_port = htons(port);
-    /* addr.sin_addr.s_addr = interface.s_addr; */
-    addr.sin_addr.s_addr = INADDR_ANY;
+    addr.sin_addr.s_addr = iface;
 
     if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
 	return -1;
     }
     if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR,
 		   (char *)&one, sizeof(one)) < 0) {
-	close(sock);
+	closesocket(sock);
 	return -1;
     }
     if (bind(sock, (struct sockaddr *)&addr, sizeof(addr)) < 0) {
-	close(sock);
+	closesocket(sock);
 	return -1;
     }
-    if (listen(sock, 5) < 0) {
-	close(sock);
+    if (listen(sock, 32) < 0) {
+	closesocket(sock);
 	return -1;
     }
 
@@ -565,9 +643,96 @@ ListenOnTCPPort(port)
 }
 
 int
-ConnectToTcpAddr(host, port)
-    char *host;
-    int port;
+rfbListenOnUnixSocket(char *file, rfbScreenInfoPtr rfbScreen)
+{
+#ifdef WIN32
+    rfbLog("Windows doesn't support UNIX sockets\n");
+    return -1;
+#else
+    int sock;
+    struct sockaddr_un addr;
+
+    if (rfbScreen) {
+	if (rfbScreen->socketState != RFB_SOCKET_READY) {
+	    rfbLog("rfbScreen is not in RFB_SOCKET_READY state.\n");
+	    return -1;
+	} else if (rfbScreen->inetdSock >= 0) {
+	    rfbLog("Cannot have inetdSock and listenSock simultaneously.\n");
+	    return -1;
+	}
+    }
+
+    if (!file || strlen(file)+1 > sizeof(addr.sun_path)) {
+	rfbLog("rfbListenOnUnixSocket: invalid filename '%s'.\n",
+	    file ? file : "null");
+	return -1;
+    }
+
+    sock = socket(AF_UNIX, SOCK_STREAM, 0);
+    if (sock < 0) {
+	rfbLogPerror("rfbListenOnUnixSocket: socket failed");
+	return -1;
+    }
+
+    memset(&addr, 0, sizeof(addr));
+    addr.sun_family = AF_UNIX;
+    strcpy(addr.sun_path, file);
+
+    unlink(file);
+
+    if (bind(sock, (struct sockaddr *)&addr, sizeof(addr)) < 0) {
+	rfbLogPerror("rfbListenOnUnixSocket: bind failed");
+	closesocket(sock);
+	return -1;
+    }
+
+    if (listen(sock, 32) < 0) {
+	rfbLogPerror("rfbListenOnUnixSocket: listen failed");
+	closesocket(sock);
+	return -1;
+    }
+
+    if (rfbScreen) {
+	if (rfbScreen->listenSock >= 0) {
+	    closesocket(rfbScreen->listenSock);
+	    FD_CLR(rfbScreen->listenSock, &rfbScreen->allFds);
+	    rfbScreen->listenSock = -1;
+	    rfbLog("Switched to listening on Unix Socket: %s\n", file);
+	} else {
+	    rfbLog("Listening on Unix Socket: %s\n", file);
+	}
+	rfbScreen->listenSock = sock;
+	FD_SET(rfbScreen->listenSock, &(rfbScreen->allFds));
+	if (rfbScreen->listenSock > rfbScreen->maxFd) {
+	    rfbScreen->maxFd = rfbScreen->listenSock;
+	}
+    }
+    return sock;
+#endif
+}
+
+int
+rfbAcceptUnixSocket(int sock)
+{
+#ifdef WIN32
+    rfbLog("Windows doesn't support UNIX sockets\n");
+    return -1;
+#else
+    struct sockaddr_un addr;
+    int fd, fromlen = sizeof(addr);
+
+    fd = accept(sock, (struct sockaddr *)&addr, &fromlen);
+    if (fd < 0) {
+	rfbLogPerror("rfbAcceptUnixSocket: accept failed");
+	return -1;
+    }
+    return fd;
+#endif
+}
+
+int
+rfbConnectToTcpAddr(char *host,
+                    int port)
 {
     struct hostent *hp;
     int sock;
@@ -577,7 +742,7 @@ ConnectToTcpAddr(host, port)
     addr.sin_family = AF_INET;
     addr.sin_port = htons(port);
 
-    if ((addr.sin_addr.s_addr = inet_addr(host)) == INADDR_NONE)
+    if ((addr.sin_addr.s_addr = inet_addr(host)) == htonl(INADDR_NONE))
     {
 	if (!(hp = gethostbyname(host))) {
 	    errno = EINVAL;
@@ -591,7 +756,7 @@ ConnectToTcpAddr(host, port)
     }
 
     if (connect(sock, (struct sockaddr *)&addr, (sizeof(addr))) < 0) {
-	close(sock);
+	closesocket(sock);
 	return -1;
     }
 
@@ -599,8 +764,8 @@ ConnectToTcpAddr(host, port)
 }
 
 int
-ListenOnUDPPort(port)
-    int port;
+rfbListenOnUDPPort(int port,
+                   in_addr_t iface)
 {
     struct sockaddr_in addr;
     int sock;
@@ -609,8 +774,7 @@ ListenOnUDPPort(port)
     memset(&addr, 0, sizeof(addr));
     addr.sin_family = AF_INET;
     addr.sin_port = htons(port);
-    /* addr.sin_addr.s_addr = interface.s_addr; */
-    addr.sin_addr.s_addr = INADDR_ANY;
+    addr.sin_addr.s_addr = iface;
 
     if ((sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
 	return -1;
@@ -625,3 +789,50 @@ ListenOnUDPPort(port)
 
     return sock;
 }
+
+/*
+ * rfbSetTcpNoDelay sets a TCP socket into TCP_NODELAY mode,
+ * and leaves other file descriptors alone. 
+ */
+rfbBool
+rfbSetTcpNoDelay(int sock)
+{
+    int one = 1;
+    struct sockaddr_in addr;
+    socklen_t addrlen = sizeof(addr);
+
+    memset(&addr, 0, sizeof(addr));
+
+    if (getsockname(sock, (struct sockaddr *)&addr, &addrlen) < 0) {
+	/* non-socket */
+	return TRUE;
+    }
+    if (addr.sin_family == AF_INET) {
+	if (setsockopt(sock, IPPROTO_TCP, TCP_NODELAY,
+		       (char *)&one, sizeof(one)) < 0) {
+	    rfbLogPerror("setsockopt TCP_NODELAY");
+	    return FALSE;
+	}
+    }
+    return TRUE;
+}
+
+/*
+ * rfbSetNonBlocking sets a socket into non-blocking mode.
+ */
+rfbBool
+rfbSetNonBlocking(int sock)
+{
+#ifdef WIN32
+  unsigned long block=1;
+  if(ioctlsocket(sock, FIONBIO, &block) == SOCKET_ERROR) {
+    errno=WSAGetLastError();
+#else
+  int flags = fcntl(sock, F_GETFL);
+  if(flags < 0 || fcntl(sock, F_SETFL, flags | O_NONBLOCK) < 0) {
+#endif
+    rfbLogPerror("Setting socket to non-blocking failed");
+    return FALSE;
+  }
+  return TRUE;
+}
