@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2002-2010 Karl J. Runge <runge@karlrunge.com> 
+   Copyright (C) 2002-2011 Karl J. Runge <runge@karlrunge.com> 
    All rights reserved.
 
 This file is part of x11vnc.
@@ -412,7 +412,7 @@ if (db24 > 2) fprintf(stderr, " check_for_multivis: %.4f\n", now - last_call);
 
 	/*
 	 * every 10 seconds we try to clean out and also refresh the window
-	 * info in the the 8bpp window table:
+	 * info in the 8bpp window table:
 	 */
 	if (now > last_clear + 10) {
 		last_clear = now;
@@ -720,7 +720,7 @@ static int check_depth_win(Window win, Window top, XWindowAttributes *attr) {
 
 	if (store_it) {
 		int i, j = -1, none = -1, nomap = -1;
-		int new = 0;
+		int newc = 0;
 		if (attr->map_state == IsViewable) {
 			/* count the visible ones: */
 			multivis_count++;
@@ -754,7 +754,7 @@ if (db24 > 1) fprintf(stderr, "multivis: 0x%lx %d\n", win, attr->depth);
 			} else if (none >= 0) {
 				/* put it in the first None slot */
 				j = none;
-				new = 1;
+				newc = 1;
 			} else if (nomap >=0) {
 				/* put it in the first unmapped slot */
 				j = nomap;
@@ -791,8 +791,8 @@ if (db24 > 1) fprintf(stderr, "multivis: STORE 0x%lx j: %3d ms: %d dep=%d\n", wi
 			windows_8bpp[j].x = x;
 			windows_8bpp[j].y = y;
 
-			if (new || now_vis) {
-if (db24) fprintf(stderr, "new/now_vis: 0x%lx %d/%d\n", win, new, now_vis);
+			if (newc || now_vis) {
+if (db24) fprintf(stderr, "new/now_vis: 0x%lx %d/%d\n", win, newc, now_vis);
 				/* mark it immediately if a new one: */
 				X_UNLOCK;	/* dont forget the giant lock */
 				mark_rect_as_modified(x, y, x + attr->width,
