@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2002-2009 Karl J. Runge <runge@karlrunge.com> 
+   Copyright (C) 2002-2011 Karl J. Runge <runge@karlrunge.com> 
    All rights reserved.
 
 This file is part of x11vnc.
@@ -44,6 +44,7 @@ static char *usage =
 "\n"
 "  x11vnc -appshare: an experiment in application sharing via x11vnc.\n"
 "\n"
+#if !SMALL_FOOTPRINT
 "  Usage:   x11vnc -appshare -id windowid -connect viewer_host:0\n"
 "           x11vnc -appshare -id pick     -connect viewer_host:0\n"
 "\n"
@@ -238,6 +239,7 @@ static char *usage =
 "     cover up existing windows that are being tracked.) See cmd=add_window\n"
 "     and cmd=add_app described above.\n"
 "\n"
+#endif
 ;
 
 #include <stdio.h>
@@ -563,6 +565,9 @@ static void be_helper_pid(char *dpy_str) {
 
 	if (ms < 50) ms = 50;
 
+#if NO_X11
+	fprintf(stderr, "be_helper_pid: not compiled with X11.\n");
+#else
 	dpy = XOpenDisplay(dpy_str);
 	ticker_atom = XInternAtom(dpy, ticker_atom_str, False);
 
@@ -583,6 +588,7 @@ static void be_helper_pid(char *dpy_str) {
 			}
 		}
 	}
+#endif
 	exit(0);
 }
 
@@ -895,7 +901,7 @@ static void delete_win(Window win) {
 static void recurse_search(int level, int level_max, Window top, Window app, int *nw) {
 	Window w, r, parent, *list = NULL;
 	unsigned int nchild;
-	int ok;
+	int ok = 0;
 
 	if (appshare_debug > 1) {
 		fprintf(stderr, "level: %d level_max: %d  top: 0x%lx  app: 0x%lx\n", level, level_max, top, app);
@@ -904,6 +910,7 @@ static void recurse_search(int level, int level_max, Window top, Window app, int
 		return;
 	}
 	
+#if !NO_X11
 	ok = XQueryTree(dpy, top, &r, &parent, &list, &nchild);
 	if (ok) {
 		int i;
@@ -930,6 +937,7 @@ static void recurse_search(int level, int level_max, Window top, Window app, int
 	if (list) {
 		XFree(list);
 	}
+#endif
 }
 		
 static void add_app(Window app) {
@@ -1350,7 +1358,7 @@ static void list_apps(void) {
 
 static int process_control(char *file, int check_clients) {
 	int i, nnew = 0, seen[CMAX];
-	char line[1024], *new[CMAX];
+	char line[1024], *newctl[CMAX];
 	FILE *f;
 
 	f = fopen(file, "r");
@@ -1473,7 +1481,7 @@ static int process_control(char *file, int check_clients) {
 			if (idx >= 0) {
 				seen[idx] = 1;
 			} else {
-				new[nnew++] = strdup(q);
+				newctl[nnew++] = strdup(q);
 			}
 		}
 	}
@@ -1501,8 +1509,8 @@ static int process_control(char *file, int check_clients) {
 				}
 				continue;
 			}
-			clients[free] = new[i];
-			client(new[i], 1);
+			clients[free] = newctl[i];
+			client(newctl[i], 1);
 		}
 	}
 	return 1;
@@ -1917,7 +1925,7 @@ int appshare_main(int argc, char *argv[]) {
 			id_opt = "-sid";
 			if (end) exiter("no -sid value supplied\n", 1);
 			app_str = strdup(argv[++i]);
-		} else if (!strcmp(s, "-connect") || !strcmp(s, "-connect_or_exit")) {
+		} else if (!strcmp(s, "-connect") || !strcmp(s, "-connect_or_exit") || !strcmp(s, "-coe")) {
 			if (end) exiter("no -connect value supplied\n", 1);
 			connect_to = strdup(argv[++i]);
 		} else if (!strcmp(s, "-control")) {
