@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2002-2009 Karl J. Runge <runge@karlrunge.com> 
+   Copyright (C) 2002-2011 Karl J. Runge <runge@karlrunge.com> 
    All rights reserved.
 
 This file is part of x11vnc.
@@ -908,6 +908,9 @@ static void tree_descend_cursor(int *depth, Window *w, win_str_info_t *winfo) {
 	*(winfo->res_name)  = '\0';
 	*(winfo->res_class) = '\0';
 
+	for (i=0; i < maxtries; i++) {
+		wins[i] = None;
+	}
 
 	/* some times a window can go away before we get to it */
 	trapped_xerror = 0;
@@ -1788,10 +1791,20 @@ int cursor_noshape_updates_clients(rfbScreenInfoPtr s) {
 	rfbClientIteratorPtr iter;
 	rfbClientPtr cl;
 	int count = 0;
+	static int calls = 0, skip = -1;
 
 	if (! s) {
 		return 0;
 	}
+	if (skip < 0) {
+		skip = 0;
+		/* for testing non-CopyRect clients... e.g. old VNC2Go */
+		if (getenv("X11VNC_SKIP_cursor_noshape_updates_clients")) skip = 1;
+	}
+	if (skip) {
+		return 0;
+	}
+
 	iter = rfbGetClientIterator(s);
 	while( (cl = rfbClientIteratorNext(iter)) ) {
 		if (!cl->enableCursorShapeUpdates) {
@@ -1799,6 +1812,9 @@ int cursor_noshape_updates_clients(rfbScreenInfoPtr s) {
 		}
 	}
 	rfbReleaseClientIterator(iter);
+	if (calls++ < 5) {
+		rfbLog("cursor_noshape_updates_clients: %d\n", count);
+	}
 	return count;
 }
 
@@ -1836,18 +1852,25 @@ void cursor_position(int x, int y) {
 		return;
 	}
 
+	if (debug_pointer > 1) rfbLog("cursor_position(1): x: %d y: %d\n", x, y);
+
 	if (scaling) {
 		x = ((double) x / dpy_x) * scaled_x;
 		x = nfix(x, scaled_x);
 		y = ((double) y / dpy_y) * scaled_y;
 		y = nfix(y, scaled_y);
+		if (debug_pointer > 1) rfbLog("cursor_position(2): x: %d y: %d dpy_x: %d dpy_y: %d scaled_x: %d scaled_y: %d\n", x, y, dpy_x, dpy_y, scaled_x, scaled_y);
 	}
 
 	if (clipshift) {
+		int udpy_x = scaling ? scaled_x : dpy_x;
+		int udpy_y = scaling ? scaled_y : dpy_y;
+
 		if (x < 0) x = 0;
 		if (y < 0) y = 0;
-		if (x >= dpy_x) x = dpy_x-1;
-		if (y >= dpy_y) y = dpy_y-1;
+		if (x >= udpy_x) x = udpy_x-1;
+		if (y >= udpy_y) y = udpy_y-1;
+		if (debug_pointer > 1) rfbLog("cursor_position(3): x: %d y: %d dpy_x: %d dpy_y: %d udpy_x: %d udpy_y: %d\n", x, y, dpy_x, dpy_y, udpy_x, udpy_y);
 	}
 
 	if (x == screen->cursorX && y == screen->cursorY) {
@@ -1989,8 +2012,13 @@ if (0) fprintf(stderr, "check_x11_pointer %d %d\n", root_x, root_y);
 	if (debug_pointer) {
 		static int last_x = -1, last_y = -1;
 		if (root_x != last_x || root_y != last_y) {
-			rfbLog("XQueryPointer:     x:%4d, y:%4d)\n",
+			rfbLog("XQueryPointer:     x:%4d, y:%4d\n",
 			    root_x, root_y);
+			if (debug_pointer > 1) {
+				rfbLog("check_x11_pointer x:%4d y:%4d "
+				    "d: %dx%d+%d+%d wd: %dx%d cd: %dx%d+%d+%d\n",
+				    root_x - off_x - coff_x, root_y - off_y - coff_y, dpy_x, dpy_y, off_x, off_y, wdpy_x, wdpy_y, cdpy_x, cdpy_y, coff_x, coff_y);
+			}
 		}
 		last_x = root_x;
 		last_y = root_y;
@@ -2005,10 +2033,12 @@ if (0) fprintf(stderr, "check_x11_pointer %d %d\n", root_x, root_y);
 		if (x < 0 || y < 0 || x >= dpy_x || y >= dpy_y)  {
 			if (cnt++ % 4 != 0) {
 				if (debug_pointer) {
-					rfbLog("Skipping cursor_position() outside our clipshift\n");
+					rfbLog("Skipping cursor_position() x:%4d y:%4d outside our "
+					    "clipshift dpy_x/y: %d/%d wdpy_x/y: %d/%d cdpy_x/y: %d/%d\n",
+					    x, y, dpy_x, dpy_y, wdpy_x, wdpy_y, cdpy_x, cdpy_y);
 				}
-				return 0;
 			}
+			return 0;
 		}
 	}
 
