@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2002-2010 Karl J. Runge <runge@karlrunge.com> 
+   Copyright (C) 2002-2011 Karl J. Runge <runge@karlrunge.com> 
    All rights reserved.
 
 This file is part of x11vnc.
@@ -57,6 +57,8 @@ int have_ssh_env(void);
 char *ipv6_getnameinfo(struct sockaddr *paddr, int addrlen);
 char *ipv6_getipaddr(struct sockaddr *paddr, int addrlen);
 int listen6(int port);
+int listen_unix(char *file);
+int accept_unix(int s);
 int connect_tcp(char *host, int port);
 int listen_tcp(int port, in_addr_t iface, int try6);
 
@@ -347,7 +349,7 @@ char *ident_username(rfbClientPtr client) {
 			if (ret > 0) {
 				int i;
 				char *q, *p;
-				for (i=0; i < sizeof(msg); i++) {
+				for (i=0; i < (int) sizeof(msg); i++) {
 					msg[i] = '\0';
 				}
 				usleep(250*1000);
@@ -415,14 +417,18 @@ int find_free_port(int start, int end) {
 	if (start <= 0) {
 		start = 1024;
 	}
-	if (end <= 0) {
+	if (end <= 0 || end > 65530) {
 		end = 65530;
 	}
 	for (port = start; port <= end; port++)  {
 		int sock = listen_tcp(port, htonl(INADDR_ANY), 0);
 		if (sock >= 0) {
 			close(sock);
-			return port;
+			sock = listen_tcp(port, htonl(INADDR_LOOPBACK), 0);
+			if (sock >= 0) {
+				close(sock);
+				return port;
+			}
 		}
 	}
 	return 0;
@@ -652,6 +658,90 @@ int listen6(int port) {
 #endif
 }
 
+#ifdef LIBVNCSERVER_HAVE_SYS_SOCKET_H
+#include <sys/un.h>
+#endif
+
+int listen_unix(char *file) {
+#if !defined(AF_UNIX) || !defined(LIBVNCSERVER_HAVE_SYS_SOCKET_H)
+	if (sock) {}
+	return -1;
+#else
+
+#ifndef LIBVNCSERVER_ALLOW_NON_AF_INET_SOCKETS
+	int sock;
+	struct sockaddr_un addr;
+
+	rfbLog("listen_unix: AF_UNIX socket %s\n", file ? file : "null");
+
+	rfbLog("listen_unix: warning: libvnserver does not support AF_UNIX sockets.\n");
+
+	if (!file || strlen(file)+1 > sizeof(addr.sun_path)) {
+		rfbLog("listen_unix: filename too long: %s\n", file);
+		return -1;
+	}
+	sock = socket(AF_UNIX, SOCK_STREAM, 0);
+	if (sock < 0) {
+		rfbLogPerror("listen_unix: socket");
+		return -1;
+	}
+
+	memset(&addr, 0, sizeof(addr));
+
+	addr.sun_family = AF_UNIX;
+	strcpy(addr.sun_path, file);
+	unlink(file);
+
+	if (bind(sock, (struct sockaddr *)&addr, sizeof(addr)) < 0) {
+		rfbLogPerror("listen_unix: bind");
+		close(sock);
+		return -1;
+	}
+
+	if (listen(sock, 32) < 0) {
+		rfbLogPerror("listen_unix: listen");
+		close(sock);
+		return -1;
+	}
+	rfbLog("linux_unix: listening on unix socket: %s fd=%d\n", file, sock);
+	return sock;
+#else
+	int sock = rfbListenOnUnixSocket(file, NULL);
+	rfbLog("listening on unix socket: %s fd=%d\n",
+	    file ? file : "null", sock);
+	return sock;
+#endif
+
+#endif
+}
+
+int accept_unix(int sock) {
+#if !defined(AF_UNIX) || !defined(LIBVNCSERVER_HAVE_SYS_SOCKET_H)
+	if (sock) {}
+	return -1;
+#else
+
+#ifndef LIBVNCSERVER_ALLOW_NON_AF_INET_SOCKETS
+	int fd, fromlen;
+	struct sockaddr_un addr;
+
+	rfbLog("accept_unix: warning: libvnserver does not support AF_UNIX sockets.\n");
+
+	fd = accept(sock, (struct sockaddr *)&addr, &fromlen);
+	if (fd < 0) {
+		rfbLogPerror("accept_unix: accept");
+		return -1;
+	}
+	return fd;
+#else
+	int fd = rfbAcceptUnixSocket(sock);
+	rfbLog("accept_unix: rfbAcceptUnixSocket(%d) returned fd=%d\n", sock, fd);
+	return fd;
+#endif
+
+#endif
+}
+
 int connect_tcp(char *host, int port) {
 	double t0 = dnow();
 	int fd = -1;
