@@ -1,6 +1,6 @@
 #!/usr/bin/env perl
 #
-# Copyright (c) 2009-2010 by Karl J. Runge <runge@karlrunge.com>
+# Copyright (c) 2009-2011 by Karl J. Runge <runge@karlrunge.com>
 #
 # ultravnc_repeater.pl is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -31,27 +31,47 @@ protocol: Listen on one port for vnc clients (default 5900.)
 
           The string "RFB 000.000\n" is sent to the client (the client
           must understand this means send ID:... or host:port.)
+
           Also accept <host>:<port> from clients and make the
           connection to the vnc server immediately. 
 
           Note there is no authentication or security WRT ID names or
           identities; it is up to the client and server to completely
-          manage that aspect and whether to encrypt the session, etc.
+          manage that aspect and whether to encrypt the subsequent VNC
+          session, etc.
 
-usage:  ultravnc_repeater.pl [-r] [client_port [server_port]]
+Usage:  ultravnc_repeater.pl [options] [client_port [server_port]]
+
+Options:
 
 Use -r to refuse new server/client connections when there is an existing
-server/client ID.  The default is to close the previous one.
+server/client ID.  The default is to close the previous one. (Same as
+setting the ULTRAVNC_REPEATER_REFUSE env. var.)
 
-To write to a log file set the env. var ULTRAVNC_REPEATER_LOGFILE.
+Use -l to set the logfile (or set the ULTRAVNC_REPEATER_LOGFILE env. var.)
+E.g. "-l /var/log/repeater.log"
 
-To run in a loop restarting the server if it exits set the env. var.
-ULTRAVNC_REPEATER_LOOP=1 or ULTRAVNC_REPEATER_LOOP=BG, the latter
-forks into the background.  Set ULTRAVNC_REPEATER_PIDFILE to a file
-to store the master pid in.
+To run an outer loop restarting the server if it exits use "-L" or "-L BG"
+(which are the same as setting the env. var. ULTRAVNC_REPEATER_LOOP=1
+or ULTRAVNC_REPEATER_LOOP=BG) The -L BG mode forks into the background.
 
-Set ULTRAVNC_REPEATER_NO_RFB=1 to disable sending "RFB 000.000" to
-the client.  Then this program acts as general TCP rendezvous tool.
+Use -p to set a file to store the master pid in for your own tracking
+(same as ULTRAVNC_REPEATER_PIDFILE).  For example, a cronjob may supply
+a -p pidfile and check if the process id stored in the file is still
+running before deciding to run this script.
+
+Use -C or set ULTRAVNC_REPEATER_CLEAN=1 to have the script periodically
+check for and close initial connections that have gone away before the
+partner connects (currently only works on Linux.)
+
+Use -R or set ULTRAVNC_REPEATER_NO_RFB=1 to disable sending "RFB 000.000"
+to the client.  Then this program acts as general TCP rendezvous tool.
+
+Set the env. var. ULTRAVNC_REPEATER_BUFSIZE to something other than 250
+bytes to modify the ultravnc proxy handshake buffer size for your own
+special application.
+
+You can also set client_port via -c and server_port via -s.
 
 Examples:
 
@@ -61,11 +81,85 @@ Examples:
 	ultravnc_repeater.pl 5901 5501
 
 	env ULTRAVNC_REPEATER_LOOP=BG ULTRAVNC_REPEATER_LOGFILE=/tmp/u.log ultravnc_repeater.pl ...
+	ultravnc_repeater.pl -L BG -l /tmp/u.log  ...
+
+Any ULTRAVNC_REPEATER_* environment variables may also be set as the initial
+arguments on the command line.
+
+This program can also be launched via web server CGI.  Please see the
+subroutine check_cgi() in the script for informtion about the query
+names and security issues.
 
 ';
 
 use strict;
 
+if (@ARGV && $ARGV[0] =~ /-h/) {
+	print $usage;
+	exit 0;
+}
+
+if ($ENV{ULTRAVNC_REPEATER_DEBUG}) {
+	print STDERR "pwd:  ", `pwd`;
+	print STDERR "zero: $0\n";
+	print STDERR "ls:   ", `ls -l $0`;
+	print STDERR "perl: ", `sh -c 'type perl'`;
+}
+
+check_cgi();
+
+my $client_port0 = 5900;
+my $server_port0 = 5500;
+
+while (@ARGV) {
+	if ($ARGV[0] =~ /^-*(ULTRAVNC_REPEATER_\w+)=(.+)/) {
+		$ENV{$1} = $2;
+		shift;
+	} elsif ($ARGV[0] eq '-l') {
+		shift;
+		$ENV{ULTRAVNC_REPEATER_LOGFILE} = shift;
+	} elsif ($ARGV[0] eq '-p') {
+		shift;
+		$ENV{ULTRAVNC_REPEATER_PIDFILE} = shift;
+	} elsif ($ARGV[0] eq '-c') {
+		shift;
+		$ENV{ULTRAVNC_REPEATER_CLIENT_PORT} = shift;
+	} elsif ($ARGV[0] eq '-s') {
+		shift;
+		$ENV{ULTRAVNC_REPEATER_SERVER_PORT} = shift;
+	} elsif ($ARGV[0] eq '-L') {
+		shift;
+		if (@ARGV && $ARGV[0] =~ /^(1|0|BG)$/) {
+			$ENV{ULTRAVNC_REPEATER_LOOP} = shift;
+		} else {
+			$ENV{ULTRAVNC_REPEATER_LOOP} = '1';
+		}
+	} elsif ($ARGV[0] eq '-R') {
+		shift;
+		if (@ARGV && $ARGV[0] =~ /^(1|0)$/) {
+			$ENV{ULTRAVNC_REPEATER_NO_RFB} = shift;
+		} else {
+			$ENV{ULTRAVNC_REPEATER_NO_RFB} = '1';
+		}
+	} elsif ($ARGV[0] eq '-C') {
+		shift;
+		if (@ARGV && $ARGV[0] =~ /^(1|0)$/) {
+			$ENV{ULTRAVNC_REPEATER_CLEAN} = shift;
+		} else {
+			$ENV{ULTRAVNC_REPEATER_CLEAN} = '1';
+		}
+	} elsif ($ARGV[0] eq '-r') {
+		shift;
+		if (@ARGV && $ARGV[0] =~ /^(1|0)$/) {
+			$ENV{ULTRAVNC_REPEATER_REFUSE} = shift;
+		} else {
+			$ENV{ULTRAVNC_REPEATER_REFUSE} = '1';
+		}
+	} else {
+		last;
+	}
+}
+
 # Set up logging:
 #
 if (exists $ENV{ULTRAVNC_REPEATER_LOGFILE}) {
@@ -95,6 +189,8 @@ sub get_out {
 	exit 0;
 }
 
+# logging printer:
+#
 sub lprint {
 	print STDERR scalar(localtime), ": ", @_, "\n";
 }
@@ -123,7 +219,7 @@ sub open_pidfile {
 ####################################################################
 # Set ULTRAVNC_REPEATER_LOOP=1 to have this script create an outer loop
 # restarting itself if it ever exits.  Set ULTRAVNC_REPEATER_LOOP=BG to
-# do this in the background as a daemon.
+# do this in the background as a daemon.  Same as -L option.
 
 if (exists $ENV{ULTRAVNC_REPEATER_LOOP}) {
 	my $csl = $ENV{ULTRAVNC_REPEATER_LOOP};
@@ -150,6 +246,7 @@ if (exists $ENV{ULTRAVNC_REPEATER_LOOP}) {
 			close STDERR;
 		}
 	}
+	# delete it so children will not do the same thing:
 	delete $ENV{ULTRAVNC_REPEATER_LOOP};
 
 	if (exists $ENV{ULTRAVNC_REPEATER_PIDFILE}) {
@@ -164,6 +261,9 @@ if (exists $ENV{ULTRAVNC_REPEATER_LOOP}) {
 		} elsif ($looppid) {
 			wait;
 		} else {
+			if ($ENV{ULTRAVNC_REPEATER_DEBUG}) {
+				print STDERR "exec $0 ", join(' ', @ARGV), "\n";
+			}
 			exec $0, @ARGV;	
 			exit 1;
 		}
@@ -196,26 +296,31 @@ my %ID;
 my $refuse = 0;
 my $init_timeout = 5;
 
-if (@ARGV && $ARGV[0] =~ /-h/) {
-	print $usage;
-	exit 0;
-}
-if (@ARGV && $ARGV[0] eq '-r') {
+my $select_timeout = 15;
+
+if (exists $ENV{ULTRAVNC_REPEATER_REFUSE} && $ENV{ULTRAVNC_REPEATER_REFUSE}) {
 	$refuse = 1;
 	lprint("enabling refuse mode (-r).");
-	shift;
 }
 
 my $client_port = shift;
 my $server_port = shift;
 
-$client_port = 5900 unless $client_port;
-$server_port = 5500 unless $server_port;
+if (exists $ENV{ULTRAVNC_REPEATER_CLIENT_PORT}) {
+	$client_port0 = $ENV{ULTRAVNC_REPEATER_CLIENT_PORT};
+}
+if (exists $ENV{ULTRAVNC_REPEATER_SERVER_PORT}) {
+	$server_port0 = $ENV{ULTRAVNC_REPEATER_SERVER_PORT};
+}
+
+$client_port = $client_port0 unless $client_port;
+$server_port = $server_port0 unless $server_port;
 
 my $uname = `uname`;
+chomp $uname;
 
 my $repeater_bufsize = 250;
-$repeater_bufsize = $ENV{BUFSIZE} if exists $ENV{BUFSIZE};
+$repeater_bufsize = $ENV{ULTRAVNC_REPEATER_BUFSIZE} if exists $ENV{ULTRAVNC_REPEATER_BUFSIZE};
 
 my ($RIN, $WIN, $EIN, $ROUT);
 
@@ -308,7 +413,8 @@ sub alarm_handler {
 	$got_alarm = 1;
 }
 
-while (my @ready = $select->can_read()) {
+while (1) {
+	my @ready = $select->can_read($select_timeout);
 	foreach my $fh (@ready) {
 		if (($client_listen && $fh == $client_listen) || ($client_listen6 && $fh == $client_listen6)) {
 			lprint("new vnc client connecting."); 
@@ -359,6 +465,20 @@ while (my @ready = $select->can_read()) {
 			do_new_server($sock, $buf);
 		}
 	}
+	clean_connections();
+}
+
+sub clean_connections {
+	if (exists $ENV{ULTRAVNC_REPEATER_CLEAN} && $ENV{ULTRAVNC_REPEATER_CLEAN}) {
+		foreach my $id (keys %ID) {
+			next if !exists $ID{$id}{sock};
+			if (!established($ID{$id}{sock})) {
+				lprint("socket for ID:$id is no longer established, closing it.");
+				close $ID{$id}{sock};
+				delete $ID{$id};
+			}
+		}
+	}
 }
 
 sub do_new_client {
@@ -725,6 +845,11 @@ sub xfer_both {
 	}
 }
 
+sub fsleep {
+	my ($time) = @_;
+	select(undef, undef, undef, $time) if $time;
+}
+
 sub cleanup {
 	close $client_listen  if $client_listen;
 	close $client_listen6 if $client_listen6;
@@ -734,3 +859,55 @@ sub cleanup {
 		close $ID{$id}{sock};
 	}
 }
+
+sub url_decode {
+	# for CGI mode
+	foreach (@_) {
+		tr/+/ /;
+		s/%([\dA-F][\dA-F])/pack("c",hex($1))/ige;
+	}
+	@_;
+}
+
+sub check_cgi {
+	# check if called via CGI launcher
+	my $request = '';
+	if (!exists $ENV{REQUEST_METHOD}) {
+		;
+	} elsif ($ENV{REQUEST_METHOD} eq 'POST' && exists $ENV{CONTENT_LENGTH}) {
+		read(STDIN, $request, $ENV{CONTENT_LENGTH});
+	} elsif ($ENV{REQUEST_METHOD} eq 'GET'  && exists $ENV{QUERY_STRING}) {
+		$request = $ENV{QUERY_STRING};
+	}
+	delete $ENV{REQUEST_METHOD} if exists $ENV{REQUEST_METHOD};
+
+	return if $request =~ /^\s*$/;
+
+	my %req = &url_decode(split(/[&=]/, $request));
+
+	print "Content-type: text/html\n\n";
+
+	if (exists $req{help} && $req{help} =~ /^(true|1)$/i) {
+		print $usage;
+		exit 0;
+	}
+
+	# WARNING: There may be a CGI security problem here letting
+	# a remote user set logfile, pidfile, and perhaps the others.
+	# Consider the user the CGI process runs as.  Can they write to
+	# important files?  Do the following checks and open() calls
+	# prevent running arbitrary commands?  Please audit for your
+	# configuration and add extra checks.  Ideally use of this script
+	# would be username and  password protected.  Consider limiting
+	# logfile and pidfile to reside in a fixed directory and/or not
+	# letting the remote user set them.
+	#
+	$ENV{ULTRAVNC_REPEATER_LOGFILE} = $req{logfile} if exists $req{logfile} && $req{logfile} !~ /[|\/<>&*+]/;
+	$ENV{ULTRAVNC_REPEATER_PIDFILE} = $req{pidfile} if exists $req{pidfile} && $req{pidfile} !~ /[|\/<>&*+]/;
+	$ENV{ULTRAVNC_REPEATER_CLIENT_PORT} = $req{cport} if exists $req{cport} && $req{cport} =~ /^\d+$/;
+	$ENV{ULTRAVNC_REPEATER_SERVER_PORT} = $req{sport} if exists $req{sport} && $req{sport} =~ /^\d+$/;
+	$ENV{ULTRAVNC_REPEATER_LOOP} = $req{loop} if exists $req{loop} && $req{loop} =~ /^(BG|1|0)$/;
+	$ENV{ULTRAVNC_REPEATER_NO_RFB} = 1 if exists $req{norfb} && $req{norfb} =~ /^(true|1)$/i;
+	$ENV{ULTRAVNC_REPEATER_CLEAN} = 1 if exists $req{clean} && $req{clean} =~ /^(true|1)$/i;
+	$ENV{ULTRAVNC_REPEATER_REFUSE} = 1 if exists $req{refuse} && $req{refuse} =~ /^(true|1)$/i;
+}
