@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2002-2010 Karl J. Runge <runge@karlrunge.com> 
+   Copyright (C) 2002-2011 Karl J. Runge <runge@karlrunge.com> 
    All rights reserved.
 
 This file is part of x11vnc.
@@ -41,6 +41,7 @@ so, delete this exception statement from your version.
 extern int debug;
 
 extern char *use_dpy;
+extern char *tag_str;
 extern int display_N;
 extern int auto_port;
 extern char *auth_file;
@@ -115,6 +116,10 @@ extern int no_external_cmds;
 extern char *allowed_external_cmds;
 extern int started_as_root;
 extern int host_lookup;
+extern char *unix_sock;
+extern char *unix_sock0;
+extern int unix_sock_only;
+extern int unix_sock_fd;
 extern int ipv6_listen;
 extern int got_ipv6_listen;
 extern int ipv6_listen_fd;
@@ -184,6 +189,10 @@ extern int macosx_console;
 extern int macosx_swap23;
 extern int macosx_resize;
 extern int macosx_icon_anim_time;
+extern int macosx_no_opengl;
+extern int macosx_no_rawfb;
+extern int macosx_read_opengl;
+extern int macosx_read_rawfb;
 
 extern unsigned long subwin;
 extern int subwin_wait_mapped;
