@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2002-2010 Karl J. Runge <runge@karlrunge.com> 
+   Copyright (C) 2002-2011 Karl J. Runge <runge@karlrunge.com> 
    All rights reserved.
 
 This file is part of x11vnc.
@@ -620,24 +620,24 @@ static void pipe_pointer(int mask, int x, int y, rfbClientPtr client) {
 	if (mask == button_mask) {
 		strcat(hint, "None");
 	} else {
-		int i, old, new, m = 1, cnt = 0;
+		int i, old, newb, m = 1, cnt = 0;
 		for (i=0; i<MAX_BUTTONS; i++) {
 			char s[20];
 
 			old = button_mask & m;
-			new = mask & m;
+			newb = mask & m;
 			m = m << 1;
 
-			if (old == new) {
+			if (old == newb) {
 				continue;
 			}
 			if (hint[0] != '\0') {
 				strcat(hint, ",");
 			}
-			if (new && ! old) {
+			if (newb && ! old) {
 				sprintf(s, "ButtonPress-%d", i+1);
 				cnt++;
-			} else if (! new && old)  {
+			} else if (! newb && old)  {
 				sprintf(s, "ButtonRelease-%d", i+1);
 				cnt++;
 			}
