@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2002-2010 Karl J. Runge <runge@karlrunge.com> 
+   Copyright (C) 2002-2011 Karl J. Runge <runge@karlrunge.com> 
    All rights reserved.
 
 This file is part of x11vnc.
@@ -2961,7 +2961,17 @@ static void nap_check(int tile_cnt) {
 	now = time(NULL);
 
 	if (screen_blank > 0) {
-		int dt_ev, dt_fbu, ms = 2000;
+		int dt_ev, dt_fbu;
+		static int ms = 0;
+		if (ms == 0) {
+			ms = 2000;
+			if (getenv("X11VNC_SB_FACTOR")) {
+				ms = ms * atof(getenv("X11VNC_SB_FACTOR"));
+			}
+			if (ms <= 0) {
+				ms = 2000;
+			}
+		}
 
 		/* if no activity, pause here for a second or so. */
 		dt_ev  = (int) (now - last_event);
@@ -3550,7 +3560,7 @@ int scan_for_updates(int count_only) {
 			fb_copy_in_progress = 0;
 			SCAN_FATAL(cs);
 			if (use_threads && pointer_mode != 1) {
-				pointer(-1, 0, 0, NULL);
+				pointer_event(-1, 0, 0, NULL);
 			}
 			nap_check(tile_count);
 			return tile_count;
@@ -3637,7 +3647,7 @@ if (tile_count) fprintf(stderr, "XX copytile: %.4f  tile_count: %d\n", dnow() -
 		 * tell the pointer handler it can process any queued
 		 * pointer events:
 		 */
-		pointer(-1, 0, 0, NULL);
+		pointer_event(-1, 0, 0, NULL);
 	}
 
 	if (blackouts) {
