@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2002-2009 Karl J. Runge <runge@karlrunge.com> 
+   Copyright (C) 2002-2011 Karl J. Runge <runge@karlrunge.com> 
    All rights reserved.
 
 This file is part of x11vnc.
@@ -1180,9 +1180,11 @@ static void handle_one_http_request(void) {
 	screen->port = 0;
 
 	http_connections(1);
+
 	rfbInitServer(screen);
 
 	if (!inetd) {
+		/* XXX ipv6 */
 		int conn = 0;
 		while (1) {
 			if (0) fprintf(stderr, "%d %d %d  %d\n", conn, screen->listenSock, screen->httpSock, screen->httpListenSock);
@@ -1234,8 +1236,7 @@ static void handle_one_http_request(void) {
 			rfbLog("handle_one_http_request: finished.\n");
 			return;
 		} else {
-			int sock = rfbConnectToTcpAddr("127.0.0.1",
-			    screen->httpPort);
+			int sock = connect_tcp("127.0.0.1", screen->httpPort);
 			if (sock < 0) {
 				exit(1);
 			}
@@ -1703,6 +1704,7 @@ static void vnc_redirect_loop(char *vnc_redirect_test, int *vnc_redirect_cnt) {
 		}
 	} else {
 		pid_t pid = 0;
+		/* XXX ipv6 */
 		if (screen->httpListenSock >= 0) {
 #if LIBVNCSERVER_HAVE_FORK
 			if ((pid = fork()) > 0) {
@@ -1817,7 +1819,7 @@ static void vnc_redirect_loop(char *vnc_redirect_test, int *vnc_redirect_cnt) {
 
 static void do_vnc_redirect(int created_disp, char *vnc_redirect_host, int vnc_redirect_port,
     int vnc_redirect_cnt, char *vnc_redirect_test) {
-	char *q = strchr(use_dpy, ':');
+	char *q = strrchr(use_dpy, ':');
 	int vdpy = -1, sock = -1;
 	int s_in, s_out, i;
 	if (vnc_redirect == 2) {
@@ -1850,7 +1852,7 @@ static void do_vnc_redirect(int created_disp, char *vnc_redirect_host, int vnc_r
 		usleep(1000*1000);
 	}
 	for (i=0; i < 20; i++) {
-		sock = rfbConnectToTcpAddr(vnc_redirect_host, vdpy);
+		sock = connect_tcp(vnc_redirect_host, vdpy);
 		if (sock >= 0) {
 			break;
 		}
@@ -1956,7 +1958,7 @@ static char *build_create_cmd(char *cmd, int *saw_xdmcp, char *usslpeer, char *t
 	char st[] = "";
 	char fdgeom[128], fdsess[128], fdopts[128], fdextra[256], fdprog[128];
 	char fdxsrv[128], fdxdum[128], fdcups[128], fdesd[128];
-	char fdnas[128], fdsmb[128], fdtag[128];
+	char fdnas[128], fdsmb[128], fdtag[128], fdxdmcpif[128];
 	char cdout[128];
 
 	if (opts) {
@@ -1980,14 +1982,18 @@ static char *build_create_cmd(char *cmd, int *saw_xdmcp, char *usslpeer, char *t
 	fdnas[0]  = '\0';
 	fdsmb[0]  = '\0';
 	fdtag[0]  = '\0';
+	fdxdmcpif[0]  = '\0';
 	cdout[0]  = '\0';
 
-	if (unixpw && keep_unixpw_opts && keep_unixpw_opts[0] != '\0') {
+	if (unixpw && keep_unixpw_opts && !getenv("X11VNC_NO_UNIXPW_OPTS")) {
 		char *q, *p, *t = strdup(keep_unixpw_opts);
+
 		if (strstr(t, "gnome")) {
 			sprintf(fdsess, "gnome");
 		} else if (strstr(t, "kde")) {
 			sprintf(fdsess, "kde");
+		} else if (strstr(t, "lxde")) {
+			sprintf(fdsess, "lxde");
 		} else if (strstr(t, "twm")) {
 			sprintf(fdsess, "twm");
 		} else if (strstr(t, "fvwm")) {
@@ -2120,8 +2126,11 @@ static char *build_create_cmd(char *cmd, int *saw_xdmcp, char *usslpeer, char *t
 	if (fdtag[0] == '\0' && getenv("FD_TAG")) {
 		snprintf(fdtag, 120, "%s", getenv("FD_TAG"));
 	}
-	if (fdxdum[0] == '\0' && getenv("FD_XDUMMY_NOROOT")) {
-		snprintf(fdxdum, 120, "%s", getenv("FD_XDUMMY_NOROOT"));
+	if (fdxdmcpif[0] == '\0' && getenv("FD_XDMCP_IF")) {
+		snprintf(fdxdmcpif,  120, "%s", getenv("FD_XDMCP_IF"));
+	}
+	if (fdxdum[0] == '\0' && getenv("FD_XDUMMY_RUN_AS_ROOT")) {
+		snprintf(fdxdum, 120, "%s", getenv("FD_XDUMMY_RUN_AS_ROOT"));
 	}
 	if (getenv("CREATE_DISPLAY_OUTPUT")) {
 		snprintf(cdout, 120, "CREATE_DISPLAY_OUTPUT='%s'", getenv("CREATE_DISPLAY_OUTPUT"));
@@ -2137,6 +2146,7 @@ static char *build_create_cmd(char *cmd, int *saw_xdmcp, char *usslpeer, char *t
 	if (strchr(fdnas, '\''))	fdnas[0] = '\0';
 	if (strchr(fdsmb, '\''))	fdsmb[0] = '\0';
 	if (strchr(fdtag, '\''))	fdtag[0] = '\0';
+	if (strchr(fdxdmcpif, '\''))	fdxdmcpif[0] = '\0';
 	if (strchr(fdxdum, '\''))	fdxdum[0] = '\0';
 	if (strchr(fdsess, '\''))	fdsess[0] = '\0';
 	if (strchr(cdout, '\''))	cdout[0] = '\0';
@@ -2151,7 +2161,8 @@ static char *build_create_cmd(char *cmd, int *saw_xdmcp, char *usslpeer, char *t
 	set_env("FD_NAS",  fdnas);
 	set_env("FD_SMB",  fdsmb);
 	set_env("FD_TAG",  fdtag);
-	set_env("FD_XDUMMY_NOROOT", fdxdum);
+	set_env("FD_XDMCP_IF",  fdxdmcpif);
+	set_env("FD_XDUMMY_RUN_AS_ROOT", fdxdum);
 	set_env("FD_SESS", fdsess);
 
 	if (usslpeer || (unixpw && keep_unixpw_user)) {
@@ -2174,7 +2185,8 @@ static char *build_create_cmd(char *cmd, int *saw_xdmcp, char *usslpeer, char *t
 		    + strlen("FD_NAS='' ")
 		    + strlen("FD_SMB='' ")
 		    + strlen("FD_TAG='' ")
-		    + strlen("FD_XDUMMY_NOROOT='' ")
+		    + strlen("FD_XDMCP_IF='' ")
+		    + strlen("FD_XDUMMY_RUN_AS_ROOT='' ")
 		    + strlen("FD_SESS='' /bin/sh ")
 		    + strlen(uu) + 1
 		    + strlen(fdgeom) + 1
@@ -2187,16 +2199,17 @@ static char *build_create_cmd(char *cmd, int *saw_xdmcp, char *usslpeer, char *t
 		    + strlen(fdnas) + 1
 		    + strlen(fdsmb) + 1
 		    + strlen(fdtag) + 1
+		    + strlen(fdxdmcpif) + 1
 		    + strlen(fdxdum) + 1
 		    + strlen(fdsess) + 1
 		    + strlen(cdout) + 1
 		    + strlen(opts) + 1);
 		sprintf(create_cmd, "env USER='%s' FD_GEOM='%s' FD_SESS='%s' "
 		    "FD_OPTS='%s' FD_EXTRA='%s' FD_PROG='%s' FD_XSRV='%s' FD_CUPS='%s' "
-		    "FD_ESD='%s' FD_NAS='%s' FD_SMB='%s' FD_TAG='%s' "
-		    "FD_XDUMMY_NOROOT='%s' %s /bin/sh %s %s",
+		    "FD_ESD='%s' FD_NAS='%s' FD_SMB='%s' FD_TAG='%s' FD_XDMCP_IF='%s' "
+		    "FD_XDUMMY_RUN_AS_ROOT='%s' %s /bin/sh %s %s",
 		    uu, fdgeom, fdsess, fdopts, fdextra, fdprog, fdxsrv,
-		    fdcups, fdesd, fdnas, fdsmb, fdtag, fdxdum, cdout, tmp, opts);
+		    fdcups, fdesd, fdnas, fdsmb, fdtag, fdxdmcpif, fdxdum, cdout, tmp, opts);
 	} else {
 		create_cmd = (char *) malloc(strlen(tmp)
 		    + strlen("/bin/sh ") + 1 + strlen(opts) + 1);
@@ -2405,6 +2418,46 @@ static void do_try_switch(char *usslpeer, char *users_list_save) {
 	}
 }
 
+static void path_lookup(char *prog) {
+	/* see create_display script */
+	char *create_display_extra = "/usr/X11R6/bin:/usr/bin/X11:/usr/openwin/bin:/usr/dt/bin:/opt/kde4/bin:/opt/kde3/bin:/opt/gnome/bin:/usr/bin:/bin:/usr/sfw/bin:/usr/local/bin";
+	char *path, *try, *p;
+	int found = 0, len = strlen(create_display_extra);
+
+	if (getenv("PATH")) {
+		len += strlen(getenv("PATH")) + 1;
+		path = (char *) malloc((len+1) * sizeof(char));
+		sprintf(path, "%s:%s", getenv("PATH"), create_display_extra);
+	} else {
+		path = (char *) malloc((len+1) * sizeof(char));
+		sprintf(path, "%s", create_display_extra);
+	}
+	try = (char *) malloc((len+2+strlen(prog)) * sizeof(char));
+
+	p = strtok(path, ":");
+	while (p) {
+		struct stat sbuf;
+
+		sprintf(try, "%s/%s", p, prog);
+		if (stat(try, &sbuf) == 0) {
+			found = 1;
+			break;
+		}
+		p = strtok(NULL, ":");
+	}
+
+	free(path);
+	free(try);
+
+	if (!found) {
+		fprintf(stderr, "\n");
+		fprintf(stderr, "The program \"%s\" could not be found in PATH and standard locations.\n", prog);
+		fprintf(stderr, "You probably need to install a package that provides the \"%s\" program.\n", prog);
+		fprintf(stderr, "Without it FINDCREATEDISPLAY mode may not be able to create an X display.\n");
+		fprintf(stderr, "\n");
+	}
+}
+
 static int do_run_cmd(char *cmd, char *create_cmd, char *users_list_save, int created_disp, int db) {
 	char tmp[] = "/tmp/x11vnc-find_display.XXXXXX";
 	char line1[1024], line2[16384];
@@ -2485,6 +2538,18 @@ static int do_run_cmd(char *cmd, char *create_cmd, char *users_list_save, int cr
 
 	rfbLog("wait_for_client: running: %s\n", cmd);
 
+	if (create_cmd != NULL) {
+		if (strstr(create_cmd, "Xvfb")) {
+			path_lookup("Xvfb");
+		}
+		if (strstr(create_cmd, "Xvnc")) {
+			path_lookup("Xvnc");
+		}
+		if (strstr(create_cmd, "Xdummy")) {
+			path_lookup("Xdummy");
+		}
+	}
+
 	if (unixpw && !unixpw_nis) {
 		int res = 0, k, j, i;
 		char line[18000];
@@ -2704,6 +2769,7 @@ if (db) fprintf(stderr, "\n");
 				clean_up_exit(1);
 			}
 			n = fread(line2, 1, 16384, p);
+			pclose(p);
 		}
 		if (tmp_fd >= 0) {
 			unlink(tmp);
@@ -2983,9 +3049,14 @@ int wait_for_client(int *argc, char** argv, int http) {
 		if (! screen->port || screen->listenSock < 0) {
 			if (got_rfbport && got_rfbport_val == 0) {
 				;
+			} else if (ipv6_listen && ipv6_listen_fd >= 0) {
+				rfbLog("Info: listening on IPv6 interface only.  (wait for client)\n");
 			} else {
 				rfbLogEnable(1);
-				rfbLog("Error: could not obtain listening port.\n");
+				rfbLog("Error: could not obtain listening port.  (wait for client)\n");
+				if (!got_rfbport && !got_ipv6_listen) {
+					rfbLog("If this system is IPv6-only, use the -6 option.\n");
+				}
 				clean_up_exit(1);
 			}
 		}
