@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2002-2010 Karl J. Runge <runge@karlrunge.com> 
+   Copyright (C) 2002-2011 Karl J. Runge <runge@karlrunge.com> 
    All rights reserved.
 
 This file is part of x11vnc.
@@ -133,6 +133,18 @@ so, delete this exception statement from your version.
  * -DPOLL_8TO24_DELAY=N  
  * -DDEBUG_XEVENTS=1  enable printout for X events.
  *
+ * -DX11VNC_MACOSX_USE_GETMAINDEVICE use deprecated GetMainDevice on macosx 
+ *
+ * -DX11VNC_MACOSX_NO_DEPRECATED_LOCALEVENTS={0,1}
+ * -DX11VNC_MACOSX_NO_DEPRECATED_POSTEVENTS={0,1}
+ * -DX11VNC_MACOSX_NO_DEPRECATED_FRAMEBUFFER={0,1}
+ *
+ * or for all:
+ *
+ * -DX11VNC_MACOSX_NO_DEPRECATED=1
+ *
+ * env. var. of the same names as above can be set to imply true.
+ *
  * Set these in CPPFLAGS before running configure. E.g.:
  *
  *   % env CPPFLAGS="-DFOREVER -DREMOTE_CONTROL=0" ./configure
@@ -246,6 +258,30 @@ so, delete this exception statement from your version.
 #define REMOTE_CONTROL 0
 #endif
 
+#ifndef X11VNC_MACOSX_NO_DEPRECATED_LOCALEVENTS
+#if     X11VNC_MACOSX_NO_DEPRECATED
+#define X11VNC_MACOSX_NO_DEPRECATED_LOCALEVENTS 1
+#else
+#define X11VNC_MACOSX_NO_DEPRECATED_LOCALEVENTS 0
+#endif
+#endif
+
+#ifndef X11VNC_MACOSX_NO_DEPRECATED_POSTEVENTS
+#if     X11VNC_MACOSX_NO_DEPRECATED
+#define X11VNC_MACOSX_NO_DEPRECATED_POSTEVENTS 1
+#else
+#define X11VNC_MACOSX_NO_DEPRECATED_POSTEVENTS 0
+#endif
+#endif
+
+#ifndef X11VNC_MACOSX_NO_DEPRECATED_FRAMEBUFFER
+#if     X11VNC_MACOSX_NO_DEPRECATED
+#define X11VNC_MACOSX_NO_DEPRECATED_FRAMEBUFFER 1
+#else
+#define X11VNC_MACOSX_NO_DEPRECATED_FRAMEBUFFER 0
+#endif
+#endif
+
 /*
  * Not recommended unless you know what you are getting into, but if you
  * define the HARDWIRE_PASSWD or HARDWIRE_VIEWPASSWD variables here or in
@@ -513,6 +549,7 @@ extern unsigned short main_red_max,   main_green_max,   main_blue_max;
 extern unsigned short main_red_shift, main_green_shift, main_blue_shift;
 
 extern int raw_fb_bytes_per_line;	/* of actual raw region we poll, not our raw_fb */
+extern int rawfb_vnc_reflect_listen;
 
 /* scaling parameters */
 extern char *scale_str;
