@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2002-2010 Karl J. Runge <runge@karlrunge.com> 
+   Copyright (C) 2002-2011 Karl J. Runge <runge@karlrunge.com> 
    All rights reserved.
 
 This file is part of x11vnc.
@@ -73,7 +73,7 @@ void initialize_xrecord(void);
 void zerodisp_xrecord(void);
 void shutdown_xrecord(void);
 int xrecord_skip_keysym(rfbKeySym keysym);
-int xrecord_skip_button(int new, int old);
+int xrecord_skip_button(int newb, int old);
 int xrecord_scroll_keysym(rfbKeySym keysym);
 void check_xrecord_reset(int force);
 void xrecord_watch(int start, int setby);
@@ -195,6 +195,9 @@ void initialize_xrecord(void) {
 	if (noxrecord) {
 		return;
 	}
+	if (remote_direct) {
+		return;
+	}
 	RAWFB_RET_VOID
 #if LIBVNCSERVER_HAVE_RECORD
 
@@ -410,9 +413,9 @@ int xrecord_skip_keysym(rfbKeySym keysym) {
 	return 0;
 }
 
-int xrecord_skip_button(int new, int old) {
+int xrecord_skip_button(int new_button, int old) {
 	/* unused vars warning: */
-	if (new || old) {}
+	if (new_button || old) {}
 
 	return 0;
 }
